/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: ParameterNode.java,v 1.10 2007/05/22 01:24:27 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.graph.system;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.xmlpull.infoset.XmlElement;
import org.xmlpull.infoset.XmlNamespace;

import edu.indiana.extreme.xbaya.graph.DataEdge;
import edu.indiana.extreme.xbaya.graph.DataPort;
import edu.indiana.extreme.xbaya.graph.Graph;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.graph.GraphSchema;
import edu.indiana.extreme.xbaya.graph.Port;
import edu.indiana.extreme.xbaya.util.XMLUtil;

/**
 * @author Satoshi Shirasuna
 */
abstract public class ParameterNode extends SystemNode {

    private static final String NAME_TAG = "name";

    private static final String DESCRIPTION_TAG = "description";

    private static final String DATA_TYPE_QNAME_TAG = "dataType";

    private static final String METADATA_TAG = "metadata";

    // private static final MLogger logger = MLogger.getLogger();

    private boolean configured;

    /**
     * Type of the parameter (e.g. xsd:string, xsd:int)
     */
    private QName parameterType;

    private String configuredName;

    private String description;

    private XmlElement metadata;

    /**
     * Constructs a ParameterNode.
     * 
     * @param nodeElement
     * @throws GraphException
     */
    public ParameterNode(XmlElement nodeElement) throws GraphException {
        super(nodeElement);
    }

    /**
     * Constructs a ParameterNode.
     * 
     * @param graph
     * 
     */
    public ParameterNode(Graph graph) {
        super(graph);
    }

    /**
     * @see edu.indiana.extreme.xbaya.graph.impl.NodeImpl#setName(java.lang.String)
     */
    @Override
    public void setName(String name) {
        super.setName(name);
        // Creates the ID to the new one based on the new name. This is
        // for readability of workflow scripts.
        createID();
    }

    /**
     * @param parameterType
     */
    public void setParameterType(QName parameterType) {
        this.parameterType = parameterType;
    }

    /**
     * Returns the type of the parameter
     * 
     * @return The type of the parameter (e.g. string, int)
     */
    public QName getParameterType() {
        return this.parameterType;
    }

    /**
     * Returns the configuredName.
     * 
     * @return The configuredName
     */
    public String getConfiguredName() {
        return this.configuredName;
    }

    /**
     * Sets configuredName.
     * 
     * @param configuredName
     *            The configuredName to set.
     */
    public void setConfiguredName(String configuredName) {
        this.configuredName = configuredName;
        setName(configuredName);
    }

    /**
     * Returns the description.
     * 
     * @return The description
     */
    public String getDescription() {
        return this.description;
    }

    /**
     * Sets description.
     * 
     * @param description
     *            The description to set.
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /**
     * @param metadata
     */
    public void setMetadata(XmlElement metadata) {
        if (metadata == null) {
            this.metadata = null;
            return;
        }

        // clone and detach from the parent
        this.metadata = XMLUtil.deepClone(metadata);

        // Reformat
        List<String> emptyTexts = new ArrayList<String>();
        for (Object child : this.metadata.children()) {
            if (child instanceof XmlElement) {
                XmlElement element = (XmlElement) child;
                for (XmlNamespace ns : element.namespaces()) {
                    // move the namespace declaration up if possible.
                    if (this.metadata.lookupNamespaceByPrefix(ns.getPrefix()) == null) {
                        // If this prefix is not used yet, copy to the root.
                        this.metadata.declareNamespace(ns);
                    }
                }
            } else if (child instanceof String) {
                String text = (String) child;
                // Remove the white spaces.
                if (text.trim().length() == 0) {
                    // We cannot remove it in the iterator.
                    emptyTexts.add(text);
                }
            }
        }
        for (String text : emptyTexts) {
            this.metadata.removeChild(text);
        }
    }

    /**
     * @return The metadata
     */
    public XmlElement getMetadata() {
        return this.metadata;
    }

    /**
     * @param configured
     */
    public void setConfigured(boolean configured) {
        this.configured = configured;
    }

    /**
     * @return true if configured; false otherwise.
     */
    public boolean isConfigured() {
        return this.configured;
    }

    /**
     * Checks if this InputNode is connected.
     * 
     * @return true if this InputNode is connected; false otherwise;
     */
    public boolean isConnected() {
        if (getEdges().size() > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Returns the port of this ParameterNode.
     * 
     * Note that a ParameterNode always has only one port.
     * 
     * @return The port
     */
    abstract public SystemDataPort getPort();

    /**
     * Returns the first port that this node is connected to/from.
     * 
     * @return The first port that this node is connected to/from
     */
    abstract public Port getConnectedPort();

    @Override
    protected void parseConfiguration(XmlElement configElement) {
        XmlElement nameElement = configElement.element(null, NAME_TAG);
        if (nameElement != null) {
            // If the name is set here, this node has been configured.
            this.configured = true;
            this.configuredName = nameElement.requiredText();
        }
        XmlElement descElement = configElement.element(null, DESCRIPTION_TAG);
        if (descElement != null) {
            this.description = descElement.requiredText();
        }
        XmlElement typeElement = configElement.element(null,
                DATA_TYPE_QNAME_TAG);
        if (typeElement != null) {
            String qnameText = typeElement.requiredText();
            if (qnameText != null && !qnameText.equals("")) {
                this.parameterType = QName.valueOf(qnameText);
            }
        }
        XmlElement metadataElement = configElement.element(null, METADATA_TAG);
        if (metadataElement != null) {
            for (XmlElement appinfo : metadataElement.requiredElementContent()) {
                // Call setMetadata to clone and reformat.
                setMetadata(appinfo);
                // It should have only one element.
                break;
            }
        }
    }

    @Override
    protected XmlElement addConfigurationElement(XmlElement nodeElement) {

        XmlElement configElement = nodeElement.addElement(GraphSchema.NS,
                GraphSchema.NODE_CONFIG_TAG);

        if (this.configured) {
            // Don't save the name here if this node has not been configured.
            XmlElement nameElement = configElement.addElement(GraphSchema.NS,
                    NAME_TAG);
            nameElement.addChild(this.configuredName);
        }
        if (this.description != null) {
            XmlElement descriptionElement = configElement.addElement(
                    GraphSchema.NS, DESCRIPTION_TAG);
            descriptionElement.addChild(this.description);
        }
        if (this.parameterType != null) {
            XmlElement qnameElement = configElement.addElement(GraphSchema.NS,
                    DATA_TYPE_QNAME_TAG);
            qnameElement.addChild(this.parameterType.toString());
        }
        if (this.metadata != null) {
            XmlElement metadataElement = configElement.addElement(
                    GraphSchema.NS, METADATA_TAG);
            // Clone the metadata to avoid parent problem because this can be
            // called multiple times.
            metadataElement.addChild(XMLUtil.deepClone(this.metadata));
        }

        return configElement;
    }

    protected List<DataEdge> getEdges() {
        DataPort port = getPort();
        List<DataEdge> edges = port.getEdges();
        return edges;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
